from pyAFL.teams.models import Team

ADE = Team("Adelaide", "adelaide")
BRI = Team("Brisbane Lions", "brisbanel")
BRB = Team("Brisbane Bears", "brisbaneb")
CAR = Team("Carlton", "carlton")
COL = Team("Collingwood", "collingwood")
ESS = Team("Essendon", "essendon")
FIZ = Team("Fitzroy", "fitzroy")
FRE = Team("Fremantle", "fremantle")
GEE = Team("Geelong", "geelong")
GC = Team("Gold Coast", "goldcoast")
GWS = Team("Greater Western Sydney", "gws")
HAW = Team("Hawthorn", "hawthorn")
MEL = Team("Melbourne", "melbourne")
NOR = Team("North Melbourne", "kangaroos")
POR = Team("Port Adelaide", "padelaide")
RIC = Team("Richmond", "richmond")
STK = Team("St Kilda", "stkilda")
SYD = Team("Sydney", "swans")
UNI = Team("University", "university")
WCE = Team("West Coast", "westcoast")
WBD = Team("Western Bulldogs", "bullldogs")

ALL_TEAMS = [
    ADE,
    BRI,
    BRB,
    CAR,
    COL,
    ESS,
    FIZ,
    FRE,
    GEE,
    GC,
    GWS,
    HAW,
    MEL,
    NOR,
    POR,
    RIC,
    STK,
    SYD,
    UNI,
    WCE,
    WBD,
]

CURRENT_TEAMS = [
    ADE,
    BRI,
    CAR,
    COL,
    ESS,
    FRE,
    GEE,
    GC,
    GWS,
    HAW,
    MEL,
    NOR,
    POR,
    RIC,
    STK,
    SYD,
    WCE,
    WBD,
]